@@ -48,17 +48,20 @@ TProgRequis Glisteprogrequis[] = {
   { CONF_STRING, "growisofspara", "", CONF_SAVEALWAYS },
   { CONF_PROG, "cdrdao", "", CONF_SAVEALWAYS },
   { CONF_STRING, "cdrdaopara", "", CONF_SAVEALWAYS },
+  { CONF_PROG, "flac", "", CONF_SAVEALWAYS },
+  { CONF_STRING, "flacpara", "", CONF_SAVEALWAYS },
   { CONF_DIR, "tmpdir", "/tmp", CONF_SAVEALWAYS },
   { CONF_STRING, "overburn", "1", CONF_SAVEALWAYS },
   { CONF_STRING, "autoblank", "1", CONF_SAVEALWAYS },
   { CONF_STRING, "fastblank", "1", CONF_SAVEALWAYS },
+  { CONF_STRING, "charsetdata", "DEFAULT", CONF_SAVEALWAYS },
   { CONF_STRING, "eject", "1", CONF_SAVEALWAYS },
   { CONF_STRING, "iconsize", "24", CONF_SAVEALWAYS },
   { CONF_STRING, "statusbar", "1", CONF_SAVEALWAYS },
   { CONF_STRING, "saveconfig", "1", CONF_SAVEALWAYS },
 
   /* onglet audio */
-  { CONF_STRING, "dstaudiocombo", "", CONF_SAVEONEXIT + CONF_COMBO },
+  { CONF_STRING, "dstaudiocombo", "", CONF_SAVEONEXIT + CONF_DEVICE },
   { CONF_STRING, "dstaudiospeed", "", CONF_SAVEONEXIT + CONF_COMBO },
   { CONF_STRING, "audiodao", "1", CONF_SAVEONEXIT + CONF_CHECK },
   { CONF_STRING, "audionotfix", "", CONF_SAVEONEXIT + CONF_CHECK },
@@ -66,7 +69,7 @@ TProgRequis Glisteprogrequis[] = {
   { CONF_STRING, "nbrcdaudio", "1", CONF_SAVEONEXIT + CONF_SPIN },
   
   /* onglet data */
-  { CONF_STRING, "dstdatacombo", "", CONF_SAVEONEXIT + CONF_COMBO },
+  { CONF_STRING, "dstdatacombo", "", CONF_SAVEONEXIT + CONF_DEVICE },
   { CONF_STRING, "dstdataspeed", "", CONF_SAVEONEXIT + CONF_COMBO },
   { CONF_STRING, "nbrcddata", "1", CONF_SAVEONEXIT + CONF_SPIN },
   { CONF_STRING, "dataformat", "", CONF_SAVEONEXIT + CONF_COMBO },
@@ -79,7 +82,7 @@ TProgRequis Glisteprogrequis[] = {
   { CONF_STRING, "datasimul", "", CONF_SAVEONEXIT + CONF_CHECK },
 
   /* onglet dvd data */
-  { CONF_STRING, "dstdvddatacombo", "", CONF_SAVEONEXIT + CONF_COMBO },
+  { CONF_STRING, "dstdvddatacombo", "", CONF_SAVEONEXIT + CONF_DEVICE },
   { CONF_STRING, "dstdvddataspeed", "", CONF_SAVEONEXIT + CONF_COMBO },
   { CONF_STRING, "nbrdvddata", "1", CONF_SAVEONEXIT + CONF_SPIN },
   { CONF_STRING, "dvddatanotfix", "", CONF_SAVEONEXIT + CONF_CHECK },
@@ -87,19 +90,20 @@ TProgRequis Glisteprogrequis[] = {
 
 
  /* onglet copier */
-  { CONF_STRING, "srccopycombo", "", CONF_SAVEONEXIT + CONF_COMBO },
-  { CONF_STRING, "dstcopycombo", "", CONF_SAVEONEXIT + CONF_COMBO },
+  { CONF_STRING, "srccopycombo", "", CONF_SAVEONEXIT + CONF_DEVICE },
+  { CONF_STRING, "dstcopycombo", "", CONF_SAVEONEXIT + CONF_DEVICE },
   { CONF_STRING, "dstcopyspeed", "", CONF_SAVEONEXIT + CONF_COMBO },
   { CONF_STRING, "nbrcdcopy", "1", CONF_SAVEONEXIT + CONF_SPIN },
   { CONF_STRING, "copysimul", "", CONF_SAVEONEXIT + CONF_CHECK },
 
   /* onglet autres */
-  { CONF_STRING, "dstothercombo", "", CONF_SAVEONEXIT + CONF_COMBO },
+  { CONF_STRING, "dstothercombo", "", CONF_SAVEONEXIT + CONF_DEVICE },
   
 #if 0
   { CONF_STRING, "extended", FALSE, "1" },
 #endif
   { CONF_STRING, "version", "1", CONF_SAVEALWAYS },
+  { CONF_STRING, "scan_drives", "0", CONF_SAVEALWAYS + CONF_PERSIST },
   { CONF_IGNORE, NULL, NULL, 0 },
 };
 
@@ -133,6 +137,17 @@ void foreachmedia(gpointer Amedia, gpointer Adata)
   fprintf(Lfic, "audiolabel=%s\n", Amediaentry->labelaudio);
 }
 
+/* callback ecriture d'un charset */
+void foreachcharsets(gpointer Acharset, gpointer Adata)
+{
+  FILE *Lfic = (FILE *) Adata;
+  Tdata *Lcharset = (Tdata *) Acharset;
+
+  fprintf(Lfic, "[charsets]\n");
+  fprintf(Lfic, "code=%s\n", Lcharset->data);
+  fprintf(Lfic, "label=%s\n", Lcharset->label);
+}
+
 /* callback ecriture d'un lecteur */
 void foreachdrives(gpointer Adrive, gpointer Adata)
 {
@@ -165,19 +180,19 @@ gboolean write_conf(GError **Aerror)
     g_warning("%s", Ltmp);
     g_free(Ltmp);
     
-    return EXIT_FAILURE;
+    return FALSE;
   }
 
   fprintf(Lfic, "[general]\n");
   fprintf(Lfic, "confversion=%s\n", VERSION);
   g_hash_table_foreach(Gconfigprog, foreachconfig, (gpointer *)(Lfic));
-
-/*  g_slist_foreach(Gmedialist, foreachmedia, (gpointer *)(fic)); */
   
   /* ecriture des lecteurs */
-
   g_slist_foreach(Glistdrives, foreachdrives, (gpointer *)(Lfic));
 
+  /* ecriture des charsets */
+  g_slist_foreach(Gcharsetlist, foreachcharsets, (gpointer *)(Lfic));
+
   fclose(Lfic);
 
   return TRUE;
@@ -333,6 +348,19 @@ gint config_find_media_callback(gconstpointer Acompsize, gconstpointer Acompvers
   return -1;
 }
 
+/* ajout d'un charset a  la liste des charsets */
+gboolean config_append_charset(gchar *Acode, gchar *Alabel)
+{
+  Tdata *Lnewcharset = g_malloc(sizeof(Tdata));
+
+  Lnewcharset->data = g_strdup(Acode);
+  Lnewcharset->label = g_strdup(Alabel && *Alabel ? Alabel : Acode);
+
+  Gcharsetlist = g_slist_append(Gcharsetlist, Lnewcharset);
+
+  return TRUE;
+}
+
 /* ajout d'un media a la liste des media */
 gboolean config_append_media(gulong Alen, gchar *Alabeldata, gchar *Alabelaudio)
 {
@@ -354,14 +382,14 @@ gboolean config_append_media(gulong Alen, gchar *Alabeldata, gchar *Alabelaudio)
 
 /* savegarde de la valeur de certains champs du formulaire lorsque
  * l'option "save on exit" est active */
-void manage_saveonexit_value(GHashTable *Ahash)
+void manage_saveonexit_value(Tgrave *Ag)
 {
   TProgRequis *Lcurentry;
   GtkWidget *Lwidget;
 
   for (Lcurentry = Glisteprogrequis; Lcurentry->entry; Lcurentry++) {
     if (!(Lcurentry->level & CONF_SAVEONEXIT)) continue;
-    Lwidget = g_hash_table_lookup(Ahash, Lcurentry->entry);
+    Lwidget = sc_grave_get_widget(Ag, Lcurentry->entry);
 
     if (!GTK_IS_WIDGET(Lwidget)) {
       g_warning("Error %s is missing!", Lcurentry->entry);
@@ -371,6 +399,9 @@ void manage_saveonexit_value(GHashTable *Ahash)
         
     if (Lcurentry->level & CONF_COMBO) {
       conf_store_value(Lcurentry->entry, get_combo_value(Lwidget));
+    } else if (Lcurentry->level & CONF_DEVICE) {
+      Tdriveinfo *Ldevice = get_combo_value(Lwidget);
+      conf_store_value(Lcurentry->entry, DRIVE_DEV(Ldevice));
     } else if (Lcurentry->level & CONF_SPIN) {
       conf_store_int(Lcurentry->entry, gtk_spin_button_get_value(GTK_SPIN_BUTTON(Lwidget)));
     } else if (Lcurentry->level & CONF_CHECK) {
@@ -380,30 +411,38 @@ void manage_saveonexit_value(GHashTable *Ahash)
 }
 
 /* gestion de la configuration (creation/chargement/sauvegarde...) */
-gboolean manage_config(GHashTable *Ahash, gshort Aop, GError **Aerror)
+gboolean manage_config(Tgrave *Ag, gshort Aop, GError **Aerror)
 {
   TProgRequis *Lcurprog;
   gchar *Lprog;
-  GtkWidget *Lwindow1;
-  Lwindow1 = g_hash_table_lookup(Ahash, "topwindow");
+  GtkWidget *Lwindow1 = sc_grave_get_widget(Ag, "topwindow");
 
   if (Aop & UPDATE_SAVEONEXIT_VALUE) {
     /* mise a jour des valeurs a sauvegarder lorsque l'on quitte */
-    manage_saveonexit_value(Ahash);
+    manage_saveonexit_value(Ag);
+  }
+
+  if (Aop & CONF_INIT_DEFAULT) {
+    /* initialsiation des valeurs par defaut */
+    for (Lcurprog = Glisteprogrequis; Lcurprog->entry; Lcurprog++) {
+      if (Lcurprog->defaultvalue && *Lcurprog->defaultvalue) {
+        Lprog = g_strdup(Lcurprog->defaultvalue);
+
+        conf_store_value(Lcurprog->entry, Lprog);
+      }
+    }
   }
 
   if (Aop & SEARCH_PROG) {
     /* on recherche les programmes externes */
     for (Lcurprog = Glisteprogrequis; Lcurprog->entry; Lcurprog++) {
-      if (Lcurprog->type == CONF_PROG) {
-        Lprog = g_find_program_in_path(Lcurprog->entry);
-      } else {
-        Lprog = g_strdup(Lcurprog->defaultvalue);
-      }
+      if (Lcurprog->type &~ CONF_PROG) continue;
+
+      Lprog = g_find_program_in_path(Lcurprog->entry);
 
       conf_store_value(Lcurprog->entry, Lprog);
 #ifdef DEBUG
-      if (! Lprog) g_warning("ERREUR LE PROG [%s] EST INDISPENSABLE ET NON TROUVE !\n", Lcurprog->entry);
+      if (! Lprog || !*Lprog) g_warning("ERREUR LE PROG [%s] EST INDISPENSABLE ET NON TROUVE !\n", Lcurprog->entry);
 #endif
     }
     
@@ -414,12 +453,18 @@ gboolean manage_config(GHashTable *Ahash, gshort Aop, GError **Aerror)
     check_sox();
     conf_store_value("sox_support_mp3", _BOOLEAN_CONF(GsupportMp3));
     conf_store_value("sox_support_ogg", _BOOLEAN_CONF(GsupportOgg));
-   
+    
+    GsupportFlac = *conf_get_string("flac");
   }
 
   if (Aop & SEARCH_LECTEUR) {
     /* maintenant scan des lecteurs */
-    cherchelecteur(NULL, Ahash);
+    cherchelecteur(NULL, Ag);
+  }
+
+  if (Aop & SEARCH_CHARSETS) {
+    /* scan des charsets definie dans mkisofs */
+    cherchecharset(NULL, Ag);
   }
 
   if (Aop & UPDATE_SIZE && GTK_IS_WIDGET(Lwindow1)) {
@@ -500,11 +545,11 @@ gboolean read_config(GError **Aerror)
   gchar *Lkey, *Lvalue;
   gchar **Llignes;
   gchar *Luneligne;
-  gchar Lemplacement[50] = "";
+  gchar Lemplacement[_BUF_SIZE] = "";
   gint i;
   gulong Lmedialen = 0;
-  gchar Lmedialabeldata[50];
-  gchar Lmedialabelaudio[50];
+  gchar Lmedialabeldata[_BUF_SIZE] = "", Lcharsetcode[_BUF_SIZE] = "";
+  gchar Lmedialabelaudio[_BUF_SIZE] = "", Lcharsetlabel[_BUF_SIZE] = "";
   Tdriveinfo *Ldriveinfo = NULL;
 
   if (!g_file_get_contents(Gfileconf, &Lcontents, NULL, Aerror)) {
@@ -540,6 +585,23 @@ gboolean read_config(GError **Aerror)
       Lkey = Luneligne;
 
       g_hash_table_insert(Gconfigprog, g_strdup(Lkey), g_strdup(Lvalue));
+    } else if (!strcmp(Lemplacement, "[charsets]")) {
+      /* section charsets */
+      Lvalue = strchr(Luneligne, '=');
+      if (Lvalue) { *(Lvalue++)=0; } else { continue; }
+      Lkey = Luneligne;
+
+      if (!strcmp(Lkey, "code")) {
+        g_strlcpy(Lcharsetcode, Lvalue, sizeof(Lcharsetcode)-1);
+      } else if (!strcmp(Lkey, "label")) {
+        g_strlcpy(Lcharsetlabel, Lvalue, sizeof(Lcharsetlabel)-1);
+      }
+
+      if (*Lcharsetcode && *Lcharsetlabel) {
+        config_append_charset(Lcharsetcode, Lcharsetlabel);
+
+        *Lcharsetcode = *Lcharsetlabel = 0;
+      }
     } else if (!strcmp(Lemplacement, "[media]")) {
       /* section liste des medias */
       Lvalue = strchr(Luneligne, '=');
@@ -598,6 +660,7 @@ gboolean read_config(GError **Aerror)
 
   GsupportMp3 = conf_get_boolean("sox_support_mp3");
   GsupportOgg = conf_get_boolean("sox_support_ogg");
+  GsupportFlac = *conf_get_string("flac");
 
   if (!test_version()) return FALSE;  
   
@@ -606,34 +669,63 @@ gboolean read_config(GError **Aerror)
 
 void maj_foreach_prog(gpointer Akey, gpointer Avalue, gpointer Adata)
 {
-  GHashTable *Lhash = (GHashTable *) Adata;
-  GtkWidget *Lobj = g_hash_table_lookup(Lhash, Akey);
-  gchar *Lname;
+  Tgrave *Lg = (Tgrave *) Adata;
+  GtkWidget *Lobj = sc_grave_get_widget(Lg, Akey);
+  gchar *Lname, *Lconfvalue;
   GtkEntry *Lobjentry;
   GtkWidget *Lobjlbl;
+  GtkWidget *Lobjtips;
+  gchar Ltipstxt[_BUF_SIZE];
+  
+  if (!GTK_IS_WIDGET(Lobj)) return;
 
-  if (!Lobj) return;
   Lname = g_strdup_printf("%s%s", gtk_widget_get_name(Lobj), "para");
-  Lobjentry = GTK_ENTRY(g_hash_table_lookup(Lhash, Lname));
-  gtk_entry_set_text(Lobjentry, *(gchar *)conf_get_string(Lname) ? conf_get_string(Lname) : "");
+  Lobjentry = GTK_ENTRY(sc_grave_get_widget(Lg, Lname));
+
+  Lconfvalue = conf_get_string(Lname);
+  gtk_entry_set_text(Lobjentry, Lconfvalue ? Lconfvalue : "");
   g_free(Lname);
 
-  Lname = g_strdup_printf("%s%s", gtk_widget_get_name(Lobj), "lbl");
-  Lobjlbl = g_hash_table_lookup(Lhash, Lname);
-  g_free(Lname);
+  Lobjlbl = sc_grave_get_widgetv(Lg, "%s%s", gtk_widget_get_name(Lobj), "lbl");
+  Lobjtips = sc_grave_get_widgetv(Lg, "%s%s", gtk_widget_get_name(Lobj), "tips");
       
   if (*(gchar *)Avalue) {
-    gtk_image_set_from_stock(GTK_IMAGE(Lobj), "gtk-yes", GTK_ICON_SIZE_MENU);
     gtk_widget_set_sensitive(GTK_WIDGET(Lobjentry), TRUE);
     gtk_label_set_text(GTK_LABEL(Lobjlbl), (gchar *)Avalue);
+
+    if (!strcmp((gchar *)Akey, "mkisofs") && FALSE == conf_get_boolean("mkisofs_support_isov2")) {
+      /* mkisofs est bien la, mais pas de support ISO v2 */
+      gtk_image_set_from_pixbuf(GTK_IMAGE(Lobj), GiconeMITIGE);
+      g_snprintf(Ltipstxt, sizeof(Ltipstxt)-1, _("%s was found but does not support ISO 9660 version 2, this function will be disabled."), (gchar *)Akey);
+    } else if (!strcmp((gchar *)Akey, "sox") &&
+        (FALSE == conf_get_boolean("sox_support_mp3") || FALSE == conf_get_boolean("sox_support_ogg"))) {
+      /* sox est bien la, mais pas de support MP3 ou/et OGG */
+      gtk_image_set_from_pixbuf(GTK_IMAGE(Lobj), GiconeMITIGE);
+      
+      if (FALSE == conf_get_boolean("sox_support_mp3") && FALSE == conf_get_boolean("sox_support_ogg")) {
+        g_snprintf(Ltipstxt, sizeof(Ltipstxt)-1, _("%s was found but does not support neither OGG or MP3 file format, those functions will be disabled."), (gchar *)Akey);
+      } else if (FALSE == conf_get_boolean("sox_support_mp3")) {
+        g_snprintf(Ltipstxt, sizeof(Ltipstxt)-1, _("%s was found but does not support MP3 file format, this function will be disabled."), (gchar *)Akey);
+      } else {
+        g_snprintf(Ltipstxt, sizeof(Ltipstxt)-1, _("%s was found but does not support OGG file format, this function will be disabled."), (gchar *)Akey);
+      }
+    } else {
+      gtk_image_set_from_pixbuf(GTK_IMAGE(Lobj), GiconeOK);
+      g_snprintf(Ltipstxt, sizeof(Ltipstxt)-1, _("%s was found."), (gchar *)Akey);
+    }
+    gtk_tooltips_set_tip(sc_grave_get_data(Lg, "_tooltips"), Lobjtips, Ltipstxt, Ltipstxt);
   } else {
-    gtk_image_set_from_stock(GTK_IMAGE(Lobj), "gtk-no", GTK_ICON_SIZE_MENU);
+    gtk_image_set_from_pixbuf(GTK_IMAGE(Lobj), GiconePASOK);
     gtk_widget_set_sensitive(GTK_WIDGET(Lobjentry), FALSE);
     gtk_label_set_text(GTK_LABEL(Lobjlbl), (gchar *)Akey);
+    g_snprintf(Ltipstxt, sizeof(Ltipstxt)-1, _("%s was not found, some features of graveman will be disabled."), (gchar *)Akey);
+    gtk_tooltips_set_tip(sc_grave_get_data(Lg, "_tooltips"), Lobjtips, Ltipstxt, Ltipstxt);
+
   }
 }
 
-void maj_proprietes_prog(GHashTable *Ahash)
+/* mise a jour de la page "programme externe" */
+void maj_proprietes_prog(Tgrave *Ag)
 {
   TProgRequis *Lcurentry;
   gchar *Lvalue;
@@ -641,7 +733,7 @@ void maj_proprietes_prog(GHashTable *Ahash)
   for (Lcurentry = Glisteprogrequis; Lcurentry->entry; Lcurentry++) {
     if (Lcurentry->type == CONF_PROG) {
       Lvalue = conf_get_string(Lcurentry->entry);
-      maj_foreach_prog(Lcurentry->entry, Lvalue, Ahash);
+      maj_foreach_prog(Lcurentry->entry, Lvalue, Ag);
     }
   }
 }
@@ -661,7 +753,7 @@ void restore_main_window_pos(GtkWidget *Awindow1)
 }
 
 /* on restaure les dernieres valeurs utilisees pour certains champs */
-void restore_last_used_values(GHashTable *Ahash)
+void restore_last_used_values(Tgrave *Ag)
 {
   TProgRequis *Lcurentry;
   GtkWidget *Lwidget;
@@ -670,7 +762,7 @@ void restore_last_used_values(GHashTable *Ahash)
     if (!(Lcurentry->level & CONF_SAVEONEXIT)) continue;
 
     if ((Lcurentry->level & CONF_SPIN) || (Lcurentry->level & CONF_CHECK)) {
-      Lwidget = g_hash_table_lookup(Ahash, Lcurentry->entry);
+      Lwidget = sc_grave_get_widget(Ag, Lcurentry->entry);
       if (!GTK_IS_WIDGET(Lwidget)) {
         g_warning("Error %s is missing!", Lcurentry->entry);
         continue;
@@ -704,7 +796,8 @@ void save_last_dir(GtkFileChooser *Awidget)
 /* test sur la version du fichier de configuration .. */
 gboolean test_version()
 {
-  if (!conf_get_string("confversion")) {
+  char *Lver = conf_get_string("confversion");
+  if (!Lver || strcmp(Lver, VERSION)) {
     gchar *Lerrmsg = g_strdup_printf(_("graveman update !\n\nYou must remove your old configuration file '%s'.\nThen you can restart graveman..\n\nThanks ! :-)"), Gfileconf);
     GtkWidget *Lerr = gtk_message_dialog_new(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING,
                           GTK_BUTTONS_OK, Lerrmsg);
