@@ -34,8 +34,8 @@
 /* communication avec growisofs */
 
 /* mise a jour du titre depuis le template prepare avec prepare_title */
-void set_title(GHashTable *Ahash, gchar *Avitesse) {
-  GtkLabel *Llabel = g_hash_table_lookup(Ahash, "gravetitle");
+void set_title(Tgrave *Ag, gchar *Avitesse) {
+  GtkLabel *Llabel = GTK_LABEL(sc_grave_get_data(Ag, "gravetitle"));
   gchar *Ltitleshow = g_strdup_printf(_("Writing DVD in progress at %sx..."), Avitesse);
 
   gtk_label_set_text(Llabel, Ltitleshow);
@@ -63,18 +63,18 @@ gchar *get_title_preparing(gint Anbrgravure, gint Acurcd, gboolean Adosimul)
 gboolean growisofs_grave_callback(GIOChannel *Astd, GIOCondition Acond, gpointer Adata)
 {
   GIOStatus Lstatus;
-  GHashTable *Lhash = (GHashTable *)Adata;
+  Tgrave *Lg = (Tgrave *)Adata;
   gchar *Lbuffer;
   gchar *s, *t;
-  GtkProgressBar *Lprogressbar = GTK_PROGRESS_BAR(g_hash_table_lookup(Lhash, "pb_total"));
-  GtkProgressBar *Lprogressbar2 = GTK_PROGRESS_BAR(g_hash_table_lookup(Lhash, "pb_step"));
-  GtkLabel *Ltitle = GTK_LABEL(g_hash_table_lookup(Lhash, "gravetitle"));
-  GError **Lerreur = (GError **) g_hash_table_lookup(Lhash, "gerror"); /* pointeur erreur */
+  GtkProgressBar *Lprogressbar = GTK_PROGRESS_BAR(sc_grave_get_data(Lg, "pb_total"));
+  GtkProgressBar *Lprogressbar2 = GTK_PROGRESS_BAR(sc_grave_get_data(Lg, "pb_step"));
+  GtkLabel *Ltitle = GTK_LABEL(sc_grave_get_data(Lg, "gravetitle"));
+  GError **Lerreur = (GError **) sc_grave_get_data(Lg, "gerror"); /* pointeur erreur */
   gdouble Lpct, Ltava;
   gchar Lsbuf[100], Lsbuf2[100];
-  gint *Lcont = (gint *) g_hash_table_lookup(Lhash, "cont"); /* on traite encore des donnees ? */
-  gint *Ltodo = (gint *) g_hash_table_lookup(Lhash, "todo"); /* nombre de piste a traiter */
-  gint *Ldone = (gint *) g_hash_table_lookup(Lhash, "done"); /* nombre de piste deja traite */
+  gint *Lcont = (gint *) sc_grave_get_data(Lg, "cont"); /* on traite encore des donnees ? */
+  gint *Ltodo = (gint *) sc_grave_get_data(Lg, "todo"); /* nombre de piste a traiter */
+  gint *Ldone = (gint *) sc_grave_get_data(Lg, "done"); /* nombre de piste deja traite */
 
   if (Acond == G_IO_HUP || Acond == G_IO_ERR) {
     *Lcont = 0;
@@ -92,13 +92,13 @@ _DEB("===>%s", Lbuffer);
     s+=strlen(GROWISOFS_CURRENT_WRITE_SPEED);
     if ((t = strchr(s, 'x'))) {
       *t = 0;
-      set_title(Lhash, s);
+      set_title(Lg, s);
     }
   } else if (!strncmp(Lbuffer, GROWISOFS_USING, strlen(GROWISOFS_USING))) {
     /* rien pour le moment */
   } else if (strstr(Lbuffer, GROWISOFS_DONE)) {
     /* gravure en cours ... */
-    if ((t=strchr(Lbuffer, '%'))) {
+    if ((t=strchr(Lbuffer, '.'))) {
       s=ltrim(Lbuffer);
 
       /* avancement tache */
@@ -123,7 +123,7 @@ _DEB("===>%s", Lbuffer);
     g_snprintf(Lsbuf2, sizeof(Lsbuf2)-1, "%.0f%%", Ltava*100);
     gtk_progress_bar_set_text(Lprogressbar, Lsbuf2);
   } else if (strstr(Lbuffer, GROWISOFS_WRITING_LEADOUT)) {
-    gboolean *Loperationstatus = (gboolean *)g_hash_table_lookup(Lhash, "operationstatus");
+    gboolean *Loperationstatus = (gboolean *)sc_grave_get_data(Lg, "operationstatus");
     gtk_label_set_text(Ltitle, _("Writing lead-out..."));
     *Loperationstatus = TRUE;
   } else if ((s=strstr(Lbuffer, GROWISOFS_FAILED))) {
@@ -139,28 +139,28 @@ _DEB("===>%s", Lbuffer);
 }
 
 /* copie d'une image iso */
-gboolean burn_dvd(GHashTable *Ahash, GError **Aerror) {
+gboolean burn_dvd(Tgrave *Ag, GError **Aerror) {
   gchar **Lcmd;
-  gchar *Lcommandline, *Lbufgrav, *Ltxt;
-  gchar **Lrepertoire = (gchar **)g_hash_table_lookup(Ahash, "tmpdir");
-  gint *Lcont = (gint *) g_hash_table_lookup(Ahash, "cont");
-  gboolean *Labort = (gboolean *) g_hash_table_lookup(Ahash, "gabort");
-  GtkWindow *Lwindow = GTK_WINDOW(g_hash_table_lookup(Ahash, "window_burn"));
-  gboolean Lsimul = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(g_hash_table_lookup(Ahash, "dvddatasimul")));
-  gint Lnbrgravure = gtk_spin_button_get_value(GTK_SPIN_BUTTON(g_hash_table_lookup(Ahash, "nbrdvddata"))); 
-  gint *Ldone = (gint *)g_hash_table_lookup(Ahash, "done"); /* fais */
+  gchar *Lcommandline, *Ltxt;
+  gchar *Lrepertoire = (gchar *)sc_grave_get_data(Ag, "tmpdir");
+  gint *Lcont = (gint *) sc_grave_get_data(Ag, "cont");
+  gboolean *Labort = (gboolean *) sc_grave_get_data(Ag, "gabort");
+  GtkWindow *Lwindow = GTK_WINDOW(sc_grave_get_data(Ag, "window_burn"));
+  gboolean Lsimul = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(sc_grave_get_widget(Ag, "dvddatasimul")));
+  gint Lnbrgravure = gtk_spin_button_get_value(GTK_SPIN_BUTTON(sc_grave_get_widget(Ag, "nbrdvddata"))); 
+  gint *Ldone = (gint *)sc_grave_get_data(Ag, "done"); /* fais */
 
-  gboolean *Loperationstatus = (gboolean *)g_hash_table_lookup(Ahash, "operationstatus");
+  gboolean *Loperationstatus = (gboolean *)sc_grave_get_data(Ag, "operationstatus");
   gboolean Lwanteject = conf_get_boolean("eject");
 
-  gint *Lpid = (gint *) g_hash_table_lookup(Ahash, "pid");
-  GtkWidget *Lvitesse = g_hash_table_lookup(Ahash, "dstdvddataspeed");
-  GtkWidget *Lgraveur = g_hash_table_lookup(Ahash, "dstdvddatacombo");
-  GtkLabel *Ltitle = GTK_LABEL(g_hash_table_lookup(Ahash, "gravetitle"));
+  gint *Lpid = (gint *) sc_grave_get_data(Ag, "pid");
+  GtkWidget *Lvitesse = sc_grave_get_widget(Ag, "dstdvddataspeed");
+  Tdriveinfo *Ldevice = get_drive_info(Ag, "dstdvddatacombo");
+  GtkLabel *Ltitle = GTK_LABEL(sc_grave_get_data(Ag, "gravetitle"));
   gboolean Leject = FALSE;
   gboolean Ldosimul;
   gchar *Lbufvitesse;
-  gchar *Loperation = g_hash_table_lookup(Ahash, "typeburn");
+  gchar *Loperation = sc_grave_get_data(Ag, "typeburn");
   gint Lcurcd;
   gint Lnbrpass=1;
   gint g_out, g_err, Lnbrarg;
@@ -169,14 +169,13 @@ gboolean burn_dvd(GHashTable *Ahash, GError **Aerror) {
   gchar *Lisopara = NULL;
   gboolean Lstatus;
 
-  GtkToggleButton *Lbtnnotfix = GTK_TOGGLE_BUTTON(g_hash_table_lookup(Ahash, "dvddatanotfix"));
+  GtkToggleButton *Lbtnnotfix = GTK_TOGGLE_BUTTON(sc_grave_get_widget(Ag, "dvddatanotfix"));
   gboolean Lnotfix = Lbtnnotfix ? gtk_toggle_button_get_active(Lbtnnotfix) : FALSE;
 _DEB("DVD DATA\n");
 _DEB("nbr gravure = [%d]\n", Lnbrgravure);
 
   Lbufvitesse = get_combo_value(Lvitesse);
-  Lbufgrav = get_combo_value(Lgraveur);
-  Lisopara = make_image_getextrapara(Ahash, "dvddata");
+  Lisopara = make_image_getextrapara(Ag, "dvddata");
 
   for (Lcurcd=1; Lcurcd<= Lnbrgravure;
       ((Lsimul && Lnbrpass>1) || (!Lsimul)) ? ( Lcurcd++, Lnbrpass=1 ) : ( Lnbrpass++ )) {
@@ -216,15 +215,15 @@ _DEB("nbr gravure = [%d]\n", Lnbrgravure);
     gtk_label_set_text(Ltitle, Ltxt);
     g_free(Ltxt);
 
-    Lcommandline = g_strdup_printf("%s %s %s %s %s%s %s -gui -use-the-force-luke=tty%s %s -f %s/",
+    Lcommandline = g_strdup_printf("%s %s %s %s %s%s %s -gui -use-the-force-luke=tty%s %s -graft-points -path-list %s",
         conf_get_string("growisofs"), 
         Lnotfix ? "" : "-dvd-compat",
-        Loperation, Lbufgrav,
+        Loperation, get_dvdrwtools_bus(Ldevice),
         *Lbufvitesse != '0' ? "-speed=" : "", *Lbufvitesse != '0' ? Lbufvitesse : "",
         conf_get_boolean("overburn") ? "-overburn" : "",
         Ldosimul ? ",dummy" : "",  /* simulation ? */
         Lisopara,  /* parametres supplementaires tel que le nom de volume du cd */
-        *Lrepertoire
+        Lrepertoire
       );
 
     _DEB("execution [%s]\n", Lcommandline);
@@ -235,7 +234,7 @@ _DEB("nbr gravure = [%d]\n", Lnbrgravure);
       break;
     }
 
-    Lstatus = g_spawn_async_with_pipes(*Lrepertoire, Lcmd, NULL, /* env argument */
+    Lstatus = g_spawn_async_with_pipes(NULL, Lcmd, NULL, /* env argument */
         (GSpawnFlags ) (G_SPAWN_DO_NOT_REAP_CHILD),
         NULL, NULL, Lpid, NULL, &g_out, &g_err, Aerror);
     g_strfreev(Lcmd);
@@ -251,13 +250,13 @@ _DEB("nbr gravure = [%d]\n", Lnbrgravure);
     g_io_channel_set_encoding (Lcom, NULL, NULL);
     g_io_channel_set_flags( Lcom, G_IO_FLAG_NONBLOCK, NULL );
     Lcomevent = g_io_add_watch (Lcom, (G_IO_IN | G_IO_HUP | G_IO_ERR | G_IO_PRI),
-                                        growisofs_grave_callback, Ahash);
+                                        growisofs_grave_callback, Ag);
   
     Lcomerr = g_io_channel_unix_new( g_err );
     g_io_channel_set_encoding (Lcomerr, NULL, NULL);
     g_io_channel_set_flags( Lcomerr, G_IO_FLAG_NONBLOCK, NULL );
     Lcomerrevent = g_io_add_watch (Lcomerr, (G_IO_IN | G_IO_HUP | G_IO_ERR | G_IO_PRI),
-                                        growisofs_grave_callback, Ahash);  
+                                        growisofs_grave_callback, Ag);  
     
     while (*Lcont>0 && *Labort == FALSE) {
       gtk_main_iteration(); 
@@ -271,6 +270,7 @@ _DEB("nbr gravure = [%d]\n", Lnbrgravure);
     g_io_channel_unref(Lcomerr);  
     g_io_channel_shutdown(Lcom, FALSE, NULL);
     g_io_channel_unref(Lcom);
+
     g_spawn_close_pid(*Lpid);
     *Lpid = 0;
     
@@ -308,7 +308,6 @@ _DEB("nbr gravure = [%d]\n", Lnbrgravure);
   }
 
   g_free(Lisopara);
-  g_free(Lbufgrav);
   g_free(Lbufvitesse);
 
 
